unit CriticalSectionUnit;

interface

uses
  Classes, MainFormUnit;

type
  TTestCriticalSection = class(TThread)
  private
    FUIString: String;
    //Two routines to simplify showing our progress
    procedure UpdateUI;
    procedure WriteString(const S: String);
  protected
    //Body of the thread
    procedure Execute; override;
    //This is the non-reentrant code
    procedure ThisIsNotReentrant;
  public
    constructor Create;
    destructor Destroy; override;
  end;

implementation

uses
  SysUtils, Windows;

var
  CritSect: TRTLCriticalSection;

constructor TTestCriticalSection.Create;
begin
  inherited Create(False);
  WriteString(Format('Thread %x started', [Handle]))
end;

destructor TTestCriticalSection.Destroy;
begin
  WriteString(Format('Thread %x terminated', [Handle]));
  inherited Destroy
end;

procedure TTestCriticalSection.UpdateUI;
begin
  Form1.Listbox1.Items.Add(FUIString)
end;

procedure TTestCriticalSection.WriteString(const S: String);
begin
  FUIString := S;
  Synchronize(UpdateUI);
end;

procedure TTestCriticalSection.Execute;
begin
  WriteString(Format(
    'Thread %x trying to enter non-reentrant code', [Handle]));
  EnterCriticalSection(CritSect);
  try
    ThisIsNotReentrant
  finally
    LeaveCriticalSection(CritSect)
  end
end;

procedure TTestCriticalSection.ThisIsNotReentrant;
begin
  WriteString(Format(
    'Thread %x entering non-reentrant code', [Handle]));
  Sleep(4000);
  WriteString(Format(
    'Thread %x leaving non-reentrant code', [Handle]));
end;

initialization
  InitializeCriticalSection(CritSect)
finalization
  DeleteCriticalSection(CritSect)
end.