unit ChildMutexUnit;

interface

uses
  SysUtils, Classes, ChildMainFormUnit;

type
  TTestChildMutex = class(TThread)
  private
    FUIString: String;
    //Two routines to simplify showing our progress
    procedure UpdateUI;
    procedure WriteString(const S: String);
  protected
    //Body of the thread
    procedure Execute; override;
    //This relies on mutually exclusive access
    procedure ThisNeedsMutualExclusiveAccess;
  public
    constructor Create;
    destructor Destroy; override;
  end;

implementation

uses
  Windows, IPCThrd;

const
  MutexName = 'SampleMutex';

constructor TTestChildMutex.Create;
begin
  inherited Create(False);
  WriteString(Format('Thread %x started', [ThreadID]))
end;

destructor TTestChildMutex.Destroy;
begin
  WriteString(Format('Thread %x terminated', [ThreadID]));
  inherited Destroy
end;

procedure TTestChildMutex.UpdateUI;
begin
  Form1.Listbox1.Items.Add(FUIString)
end;

procedure TTestChildMutex.WriteString(const S: String);
begin
  FUIString := S;
  Synchronize(UpdateUI);
end;

procedure TTestChildMutex.Execute;
var
  LocalMutex: TMutex;
begin
  try
    LocalMutex := TMutex.Create(MutexName);
    try
      if LocalMutex.Get(Infinite) then
        try
          ThisNeedsMutualExclusiveAccess
        finally
          LocalMutex.Release
        end
      else
        WriteString(Format(
          'Thread %x failed to get mutex', [Handle]));
    finally
      LocalMutex.Free;
    end;
  except
    WriteString('Mutex already created');
  end;
end;

procedure TTestChildMutex.ThisNeedsMutualExclusiveAccess;
begin
  WriteString(Format(
    'Thread %x got mutex', [ThreadID]));
  Sleep(4000);
  WriteString(Format(
    'Thread %x finished with mutex', [ThreadID]));
end;

end.