unit NetLock;

interface

uses
  WinTypes, WinProcs;

{ Core routines that mimic Win32 parameter set }
{$ifndef Win32}
function LockFile(Handle: Integer; FileOffsetLow, FileOffsetHigh,
                  LockBytesLow, LockBytesHigh: Integer): Bool;

function UnlockFile(Handle: Integer; FileOffsetLow, FileOffsetHigh,
  LockBytesLow, LockBytesHigh: Integer): Bool;
{$endif}

{ Handle based routine that un/locks a given number of bytes }
function LockFileArea(Handle: Integer; FileOffset, LockBytes: Longint;
                      Lock: Boolean): Bool;
{ File variable based routine that un/locks a given number of records }
function LockFileVarArea(var FileVar; RecordNumber, NumRecords: Longint;
                         Lock: Boolean): Bool;
{ File variable based routine that un/locks one record }
function LockFileVar(var FileVar; RecordNumber: Longint;
                     Lock: Boolean): Bool;

implementation

uses
  SysUtils;

{$ifndef Win32}
{ Helper routine }
function ApplyLock(Handle: Integer; FileOffsetLow, FileOffsetHigh,
  LockBytesLow, LockBytesHigh: Integer; Lock: Boolean): Bool; assembler;
asm
  mov ah, $5C
  mov al, 1
  sub al, Lock
  mov bx, Handle
  mov cx, FileOffsetHigh
  mov dx, FileOffsetLow
  mov si, LockBytesHigh
  mov di, LockBytesLow
  int $21
  jnc @1
  xor ax, ax
  jmp @2
 @1:
  mov ax, 1
 @2:
end;

{ Core routines that mimic Win32 parameter set }
function LockFile(Handle: Integer; FileOffsetLow, FileOffsetHigh,
  LockBytesLow, LockBytesHigh: Integer): Bool;
begin
  Result := ApplyLock(Handle, FileOffsetLow, FileOffsetHigh,
    LockBytesLow, LockBytesHigh, True);
end;

function UnlockFile(Handle: Integer; FileOffsetLow, FileOffsetHigh,
  LockBytesLow, LockBytesHigh: Integer): Bool;
begin
  Result := ApplyLock(Handle, FileOffsetLow, FileOffsetHigh,
    LockBytesLow, LockBytesHigh, False);
end;
{$endif}

{ Handle based routine that un/locks a given number of bytes }
function LockFileArea(Handle: Integer; FileOffset, LockBytes: Longint;
                      Lock: Boolean): Bool;
const
  LockFuncs: array[Boolean] of function (Handle: Integer;
    FileOffsetLow, FileOffsetHigh, LockBytesLow,
    LockBytesHigh: Integer): Bool {$ifdef Win32}stdcall{$endif}=
    (UnlockFile, LockFile);
begin
  Result := LockFuncs[Lock](Handle, LoWord(FileOffset),
      HiWord(FileOffset), LoWord(LockBytes), HiWord(LockBytes));
end;

{ File variable based routine that un/locks a given number of records }
function LockFileVarArea(var FileVar; RecordNumber, NumRecords: Longint;
                         Lock: Boolean): Bool;
begin
  with TFileRec(FileVar) do
    Result := LockFileArea(Handle, Recordnumber * RecSize,
                           NumRecords * RecSize, Lock);
end;

{ File variable based routine that un/locks one record }
function LockFileVar(var FileVar; RecordNumber: Longint;
                        Lock: Boolean): Bool;
begin
  Result := LockFileVarArea(FileVar, RecordNumber, 1, Lock);
end;

end.
